@@ -1,3 +1,4 @@
-from insolver.wrappers_v2.utils import load_model
+from .utils import load_model
 from .base import InsolverBaseWrapper
 from .glm import InsolverGLMWrapper
+from .gbm import InsolverGBMWrapper
