@@ -1,12 +1,5 @@
-import sys
-
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
-
 from os import PathLike
-from typing import Optional, Dict, Any, Union, List, Tuple, Callable
+from typing import Optional, Dict, Any, Union, List, Tuple, Callable, Literal
 
 from sklearn.base import BaseEstimator
 from sklearn.pipeline import Pipeline
@@ -23,14 +16,10 @@ from ..utils import warn_insolver
 from .base import InsolverBaseWrapper, InsolverWrapperWarning
 from .utils import save_pickle, save_dill, save_h2o
 from .utils.h2o_utils import x_y_to_h2o_frame, h2o_start, h2o_stop, to_h2oframe, load_h2o
+from .utils.hypertoptcv import hyperopt_cv_proc, tpe, rand
 
 
 class InsolverGLMWrapper(InsolverBaseWrapper):
-    algo = 'glm'
-    _backends = ["h2o", "sklearn"]
-    _tasks = ["class", "reg"]
-    _backend_saving_methods = {'sklearn': {'pickle': save_pickle, 'dill': save_dill}, 'h2o': {'h2o': save_h2o}}
-
     """Insolver wrapper for Generalized Linear Models.
 
     Parameters:
@@ -44,12 +33,17 @@ class InsolverGLMWrapper(InsolverBaseWrapper):
         **kwargs: Parameters for GLM estimators (for H2OGeneralizedLinearEstimator or TweedieRegressor) except
           `family` (`power` for TweedieRegressor) and `link`.
 
-        """
+    """
+
+    algo = 'glm'
+    _backends = ["h2o", "sklearn"]
+    _tasks = ["class", "reg"]
+    _backend_saving_methods = {'sklearn': {'pickle': save_pickle, 'dill': save_dill}, 'h2o': {'h2o': save_h2o}}
 
     def __init__(
         self,
         backend: Optional[Literal['sklearn', 'h2o']],
-        task: Optional[Literal['class', 'reg']] = 'reg',
+        task: Literal['class', 'reg'] = 'reg',
         family: Optional[str] = None,
         link: Optional[str] = None,
         h2o_server_params: Optional[Dict] = None,
@@ -69,6 +63,8 @@ class InsolverGLMWrapper(InsolverBaseWrapper):
         self.link = link
         self.h2o_server_params = h2o_server_params
         self.kwargs = kwargs
+        self.best_params: Optional[Dict[str, Any]] = None
+        self.trials = None
         self.model = self.init_model()
         self.__dict__.update(self.metadata)
 
@@ -77,12 +73,12 @@ class InsolverGLMWrapper(InsolverBaseWrapper):
 
         # Checks on supported families vs tasks
         if self.family not in [None, 'poisson', 'gamma', 'tweedie', 'normal', 'gaussian', 'inverse_gaussian', 'logit']:
-            ValueError(f'Distribution family "{self.family}" is not supported with sklearn backend.')
+            raise ValueError(f'Distribution family "{self.family}" is not supported with sklearn backend.')
         else:
             if (self.family in ['logit']) and (self.task == 'reg'):
-                ValueError(f'Distribution family "{self.family}" does not match the task "{self.task}".')
+                raise ValueError(f'Distribution family "{self.family}" does not match the task "{self.task}".')
             if (self.family not in [None, 'logit']) and (self.task == 'class'):
-                ValueError(f'Distribution family "{self.family}" does not match the task "{self.task}".')
+                raise ValueError(f'Distribution family "{self.family}" does not match the task "{self.task}".')
             if self.family is None:
                 self.family = 'gaussian' if self.task == 'reg' else 'logit'
 
@@ -149,6 +145,7 @@ class InsolverGLMWrapper(InsolverBaseWrapper):
             model = LogisticRegression(**params)
 
         if self.family in ['poisson', 'gamma', 'tweedie', 'inverse_gaussian']:
+            # Since sklearn GLM tend to fail optimization on non-standardized data
             model = Pipeline([('scaler', StandardScaler(with_mean=True, with_std=True)), ('glm', model)])
             self.metadata.update({'is_standardized': True})
         else:
@@ -160,19 +157,14 @@ class InsolverGLMWrapper(InsolverBaseWrapper):
         model = H2OGeneralizedLinearEstimator(family=self.family, link=self.link, **params)
         return model
 
-    def init_model(self) -> Any:
+    def init_model(self, additional_params: Optional[Dict] = None) -> Any:
         model = None
+        params = self.metadata['init_params']['kwargs']
+        if additional_params is not None:
+            params.update(additional_params)
         if self.backend == 'sklearn':
-            params = self.metadata['init_params']['kwargs']
-            # params.update(params.pop('kwargs'))
             model = self._init_glm_sklearn(**params)
         if self.backend == 'h2o':
-            params = {
-                key: self.metadata['init_params'][key]
-                for key in self.metadata['init_params']
-                if key not in ['family', 'link', 'backend', 'task', 'h2o_server_params', 'kwargs']
-            }
-            params.update(self.metadata['init_params']['kwargs'])
             model = self._init_glm_h2o(**params)
         self._update_metadata()
         return model
@@ -451,3 +443,53 @@ class InsolverGLMWrapper(InsolverBaseWrapper):
                 result.to_csv(path_or_buf, **kwargs)
         else:
             warn_insolver('No coefficients available!', InsolverWrapperWarning)
+
+    def hyperopt_cv(
+        self,
+        x: Union[DataFrame, Series],
+        y: Union[DataFrame, Series],
+        params: Dict[str, Any],
+        fn: Callable = None,
+        algo: Union[None, rand.suggest, tpe.suggest] = None,
+        max_evals: int = 10,
+        timeout: Optional[int] = None,
+        fmin_params: Dict[str, Any] = None,
+        fn_params: Dict[str, Any] = None,
+    ) -> Optional[Dict[str, Any]]:
+        """Hyperparameter optimization using hyperopt. Using cross-validation to evaluate hyperparameters by default.
+
+        Args:
+            x (pd.DataFrame, pd.Series): Training data.
+            y (pd.DataFrame, pd.Series): Training target values.
+            params (dict): Dictionary of hyperparameters passed to hyperopt.
+            fn (callable, optional): Objective function to optimize with hyperopt.
+            algo (callable, optional): Algorithm for hyperopt. Available choices are: hyperopt.tpe.suggest and
+             hyperopt.random.suggest. Using hyperopt.tpe.suggest by default.
+            max_evals (int, optional): Number of function evaluations before returning.
+            timeout (None, int, optional): Limits search time by parametrized number of seconds.
+             If None, then the search process has no time constraint. None by default.
+            fmin_params (dict, optional): Dictionary of supplementary arguments for hyperopt.fmin function.
+            fn_params (dict, optional):  Dictionary of supplementary arguments for custom fn objective function.
+
+        Returns:
+            dict: Dictionary of the best choice of hyperparameters. Also, best model is fitted.
+        """
+        if self.backend == 'h2o':
+            raise NotImplementedError("Method hyperopt_cv() is not supported for backend == 'h2o'")
+
+        # If model is a Pipeline, then tune parameters for its last step.
+        if isinstance(self.model, Pipeline) and ((fn_params is not None) and ("fit_params" in fn_params)):
+            fn_params["fit_params"] = {
+                f"{self.model.steps[-1][0]}__{key}": fn_params["fit_params"].get(key)
+                for key in fn_params["fit_params"].keys()
+            }
+
+        self.best_params, self.trials = hyperopt_cv_proc(
+            self, x, y, params, fn, algo, max_evals, timeout, fmin_params, fn_params
+        )
+        self._update_metadata()
+        self.model = self.init_model(self.best_params)
+        self.fit(
+            x, y, **({} if not ((fn_params is not None) and ("fit_params" in fn_params)) else fn_params["fit_params"])
+        )
+        return self.best_params
